package ru.roe.timesheets.core;

import com.google.common.base.Strings;
import org.springframework.stereotype.Component;
import ru.roe.timesheets.entity.*;
import ru.roe.timesheets.enumeration.GroupingMethod;

import java.text.SimpleDateFormat;
import java.util.Date;

@Component(BuildFilterStringBean.NAME)
public class BuildFilterStringBean {
    public static final String NAME = "timesheets_BuildFilterStringBean";

    public String getFilterString(Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, String filterTaskNamePart, GroupingMethod groupingMethod, Boolean exactFilter, Boolean nullFilter)
    {
        SimpleDateFormat sqldt = new SimpleDateFormat("dd.MM.yyyy");
        String sqlStartDate = sqldt.format(startDate);
        String sqlEndDate = sqldt.format(endDate);

        String wsWhere = "(AD.TimeByDay BETWEEN convert(date,'" + sqlStartDate + "',104) and convert(date,'" + sqlEndDate + "',104))";

        if (filterProject == null)
        {
            if (nullFilter != null) {
                if ((groupingMethod == GroupingMethod.BY_PROJECTS) && (nullFilter)) {
                    if (wsWhere != "")
                        wsWhere = wsWhere + " and ";
                    wsWhere = wsWhere + "(A.ProjectUID is null) ";
                }
            }
        }
        else
        {
            if (!Strings.isNullOrEmpty(filterProject.getOwnId())) {
                if (wsWhere != "")
                    wsWhere = wsWhere + " and ";
                wsWhere = wsWhere + "(A.ProjectUID = '" + filterProject.getOwnId() + "')";
            }
        }

        if (filterResource == null)
        {
            if (nullFilter != null) {
                if ((groupingMethod == GroupingMethod.BY_RESOURCE) && (nullFilter)) {
                    if (wsWhere != "")
                        wsWhere = wsWhere + " and ";
                    wsWhere = wsWhere + "(A.ResourceUID is null) ";
                }
            }
        }
        else
        {
            if (!Strings.isNullOrEmpty(filterResource.getOwnId())) {
                if (wsWhere != "")
                    wsWhere = wsWhere + " and ";
                wsWhere = wsWhere + "(A.ResourceUID = '" + filterResource.getOwnId() + "')";
            }
        }

        if (filterLifeCyclePart == null)
        {
            if (nullFilter != null) {
                if ((groupingMethod == GroupingMethod.BY_LIFE_CYCLE_PART) && (nullFilter)) {
                    if (wsWhere != "")
                        wsWhere = wsWhere + " and ";
                    wsWhere = wsWhere + "(MSP_EpmLookupTable2.MemberUID is null) ";
                }
            }
        }
        else
        {
            if (!Strings.isNullOrEmpty(filterLifeCyclePart.getOwnId())) {
                if (wsWhere != "")
                    wsWhere = wsWhere + " and ";
                wsWhere = wsWhere + "(MSP_EpmLookupTable2.MemberUID = '" + filterLifeCyclePart.getOwnId() + "')";
            }
        }

        if (filterSystem == null) {
            if (nullFilter != null) {
                if ((groupingMethod == GroupingMethod.BY_SYSTEM) && (nullFilter)) {
                    if (wsWhere != "")
                        wsWhere = wsWhere + " and ";
                    wsWhere = wsWhere + "(MSP_EpmLookupTable3.MemberUID is null) ";
                }
            }
        }
        else
        {
            if (!Strings.isNullOrEmpty(filterSystem.getOwnId())) {
                if (wsWhere != "")
                    wsWhere = wsWhere + " and ";
                wsWhere = wsWhere + "((MSP_EpmLookupTable3.MemberUID = '" + filterSystem.getOwnId() + "') ";
                if (!((groupingMethod == GroupingMethod.BY_SYSTEM) && (exactFilter)))
                    wsWhere = wsWhere + "or (MSP_EpmLookupTable3.MemberUID in (select memberUid from MSP_EpmLookupTable " +
                        "where (MSP_EpmLookupTable.LookupTableUID = '7B1E149C-621A-4AC5-A496-2642B8A04230') " +
                        "and (parentmemberUid = '" + filterSystem.getOwnId() + "')))";
                wsWhere = wsWhere + ")";
            }
        }

        if (filterWorkType == null)
        {
            if (nullFilter != null) {
                if ((groupingMethod == GroupingMethod.BY_WORK_TYPE) && (nullFilter)) {
                    if (wsWhere != "")
                        wsWhere = wsWhere + " and ";
                    wsWhere = wsWhere + "(MSP_EpmLookupTable4.MemberUID is null) ";
                }
            }
        }
        else
        {
            if (!Strings.isNullOrEmpty(filterWorkType.getOwnId())) {
                if (wsWhere != "")
                    wsWhere = wsWhere + " and ";
                wsWhere = wsWhere + "((MSP_EpmLookupTable4.MemberUID = '" + filterWorkType.getOwnId() + "') ";
                if (!((groupingMethod == GroupingMethod.BY_WORK_TYPE) && (exactFilter)))
                    wsWhere = wsWhere + "or (MSP_EpmLookupTable4.MemberUID in (select memberUid from MSP_EpmLookupTable " +
                        "where (MSP_EpmLookupTable.LookupTableUID = 'A44F3EF1-83DF-4A9C-AECA-AB41B884D323') " +
                        "and (parentmemberUid = '" + filterWorkType.getOwnId() + "')))";
                wsWhere = wsWhere + ")";
            }
        }

        if (!Strings.isNullOrEmpty(filterTaskNamePart))
        {
            if (wsWhere != "")
                wsWhere = wsWhere + " and ";
            wsWhere = wsWhere + "(T.TaskName like '%" + filterTaskNamePart + "'%)";
        }

        if (wsWhere != "")
            wsWhere = "where " + wsWhere;

        return wsWhere;
    }
}