package ru.roe.timesheets.core;

import org.apache.commons.lang3.time.DateUtils;
import org.springframework.stereotype.Component;
import ru.roe.timesheets.enumeration.TimeScaleUnit;

import java.text.SimpleDateFormat;
import java.time.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Component(TimeScaleBean.NAME)
public class TimeScaleBean {
    public static final String NAME = "itdogovorolapdemo_TimeScaleBean";

    // получение имени периода по начальной дате и типу периода
    public static String getPeriodName(TimeScaleUnit unit, Date startDate) {
        String result = "";
        LocalDate startLocalDate = new java.sql.Date(startDate.getTime()).toLocalDate();
        switch (unit) {
            case DAY:
                SimpleDateFormat dtd = new SimpleDateFormat("dd.MM.yyyy");
                result = dtd.format(startDate);
                break;

            case WEEK:
                Calendar calendar = Calendar.getInstance();
                calendar.set(startDate.getYear(), startDate.getMonth(), startDate.getDate());

                SimpleDateFormat dtForWeek = new SimpleDateFormat("W");
                String weekNumber = dtForWeek.format(calendar.getTime());
                Integer weekN = Integer.parseInt(weekNumber);

                calendar.set(calendar.getTime().getYear(), calendar.getTime().getMonth(), 1);
                weekNumber = dtForWeek.format(calendar.getTime());

                System.out.println("Period: " + startDate.toString() + " first date: " + calendar.getTime().toString() + " week number of first date: " + weekNumber + " weekN: " + weekN);

                if (Integer.parseInt(weekNumber) == 0)
                    weekN = weekN + 1;

                SimpleDateFormat dtw = new SimpleDateFormat(" нед. MMMMM");
                result = weekN + dtw.format(startDate);
                break;

            case MONTH:
                SimpleDateFormat dtm = new SimpleDateFormat("LLLLL yyyy");
                result = dtm.format(startDate);
                break;

            case QUARTER:
                int currentMonth = startLocalDate.getMonthValue();
                if (currentMonth <= 3) {    // 1 квартал
                    result = "1 кв. " + startLocalDate.getYear() + "г.";
                } else if (currentMonth <= 6) { // 2 квартал
                    result = "2 кв. " + startLocalDate.getYear() + "г.";
                } else if (currentMonth <= 9) { // 3 квартал
                    result = "3 кв. " + startLocalDate.getYear() + "г.";
                } else if (currentMonth <= 12) { // 4 квартал
                    result = "4 кв. " + startLocalDate.getYear() + "г.";
                }
                break;

            case YEAR:
                SimpleDateFormat dty = new SimpleDateFormat("yyyy");
                result = dty.format(startDate);
                break;
            default:
        }
        return result;
    }


    // автоматический подбор TimeScaleUnit в зависимости от выбранного интервала дат
    public static TimeScaleUnit getDefaultTimeScaleUnitForInterval(Date startDate, Date endDate) {
        if (startDate == null) return null;
        if (endDate == null) return null;
        if (startDate.after(endDate)) return null;
        LocalDate startLocalDate = new java.sql.Date(startDate.getTime()).toLocalDate();
        LocalDate endLocalDate = new java.sql.Date(endDate.getTime()).toLocalDate();
        java.time.Period period = java.time.Period.between(startLocalDate, endLocalDate);
        TimeScaleUnit returnValue = TimeScaleUnit.MONTH;
        long totalMonths = period.toTotalMonths();
        if(totalMonths < 1){
            returnValue = TimeScaleUnit.DAY;
        } else if(totalMonths <= 2){
            returnValue = TimeScaleUnit.WEEK;
        } else if(totalMonths < 12){
            returnValue = TimeScaleUnit.MONTH;
        } else if(totalMonths <= 24){
            returnValue = TimeScaleUnit.QUARTER;
        } else if(totalMonths > 24){
            returnValue = TimeScaleUnit.YEAR;
        }
        return returnValue;
    }


    // получить дату окончания периода по начальной дате и типу периода
    public static Date getEndDate(TimeScaleUnit unit, Date startDate) {
        Date resultDate = new Date();
        LocalDate startLocalDate = new java.sql.Date(startDate.getTime()).toLocalDate();
        switch (unit) {
            case DAY:
                return startDate;
            case WEEK:
                return java.sql.Date.valueOf(startLocalDate.plusWeeks(1).minusDays(1));
            case MONTH:
                return java.sql.Date.valueOf(startLocalDate.plusMonths(1).minusDays(1));
            case QUARTER:
                return java.sql.Date.valueOf(startLocalDate.plusMonths(3).minusDays(1));
            case YEAR:
                return java.sql.Date.valueOf(startLocalDate.plusYears(1).minusDays(1));
            default:
        }
        return resultDate;
    }


    // получение момента окончания текущей даты
    public static Date getDateEnd(Date currentDate) {
        LocalDate startDate = new java.sql.Date(currentDate.getTime()).toLocalDate();
        LocalDateTime startTime = startDate.atTime(23, 59, 59);
        return Date.from(startTime.atZone(ZoneId.systemDefault()).toInstant());
    }


    // вернуть список временных инетрвалов указанного типа внутри заданного периода
    public static List<ru.roe.timesheets.utils.Period> getTimePeriodsByUnit(Date startDate, Date endDate, TimeScaleUnit unit) {
//        System.out.println("**************************getTimePeriodsByUnit*************************************");
//        System.out.println("*** unit = " + unit + "****");

        if (startDate == null) return null;
        if (endDate == null) return null;
        if (unit == null) return null;
        if (startDate.after(endDate)) return null;

        LocalDate start = new java.sql.Date(startDate.getTime()).toLocalDate();
        LocalDate end = new java.sql.Date(endDate.getTime()).toLocalDate();
        List<ru.roe.timesheets.utils.Period> result = new ArrayList<>();
        LocalDate localDateD1;
        LocalDate localDateD2;
        LocalDate localDateP1;
        LocalDate localDateP2;

        switch (unit) {
            case DAY:
                for (LocalDate date = start; date.isBefore(end); date = date.plusDays(1)) {
                    setOutputItem(result, date, date, date, date);
                }
                setOutputItem(result, end, end, end, end);    // добавляем последний день временного интервала
                break;

            case WEEK:
                localDateD1 = localDateD2 = localDateP2 = start;
                localDateP1 = start.with(DayOfWeek.MONDAY); // понедельник
                while (localDateD2.getDayOfWeek().getValue() != 6) {
                    localDateD2 = localDateP2 = localDateD2.plusDays(1);  // получаем ближайшее воскресенье по номеру дня недели
                }
                if (localDateD2.isAfter(end)) {   // если временной интервал Уже, чем юнит
                    localDateP2 = localDateD2;
                    localDateD2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);    // записываем первую неделю из интервала
                    break;
                }
                setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);    // записываем первую неделю из интервала
                for (localDateD1 = localDateD2.plusDays(1); localDateD1.isBefore(end); localDateD1 = localDateD1.plusWeeks(1)) {
                    localDateP1 = localDateD1;
                    localDateD2 = localDateP2 = localDateD1.plusWeeks(1).minusDays(1);  // шаг недели
                    if (localDateD2.isAfter(end)) {
                        localDateP2 = localDateD2;
                        localDateD2 = end;
                    }
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);
                }
                if (localDateD1.isEqual(end)) {
                    localDateD1 = localDateD2 = localDateP2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);    // добавляем последний день недели
                }
                break;

            case MONTH:
                localDateD1 = start;
                localDateP1 = start.withDayOfMonth(1); // первый день месяца
                localDateD2 = localDateP2 = localDateD1.withDayOfMonth(1).plusMonths(1).minusDays(1);
                if (localDateD2.isAfter(end)) {   // если временной интервал Уже, чем юнит
                    localDateP2 = localDateD2;
                    localDateD2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);  // записываем первый месяц из интервала
                    break;
                }
                setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);  // записываем первый месяц из интервала
                for (localDateD1 = localDateD2.plusDays(1); localDateD1.isBefore(end); localDateD1 = localDateD1.plusMonths(1)) {
                    localDateP1 = localDateD1;
                    localDateD2 = localDateP2 = localDateD1.plusMonths(1).minusDays(1);
                    if (localDateD2.isAfter(end)) {
                        localDateP2 = localDateD2;
                        localDateD2 = end;
                    }
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);
                }
                if (localDateD1.isEqual(end)) {
                    localDateD1 = localDateD2 = localDateP2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);    // добавляем последний день месяца
                }
                break;

            case QUARTER:
                localDateD1 = localDateD2 = localDateP1 = localDateP2 = start;
                int month = start.getMonthValue();
                if (month <= 3){    // 1 квартал
                    localDateD2 = localDateP2 = LocalDate.of(localDateD1.getYear(), 3, 31); // 31 марта
                    localDateP1 = LocalDate.of(localDateD1.getYear(), 1, 1);; // 1 января
                } else if (month <= 6){ // 2 квартал
                    localDateD2 = localDateP2 = LocalDate.of(localDateD1.getYear(), 6, 30); // 30 июня
                    localDateP1 = LocalDate.of(localDateD1.getYear(), 4, 1);; // 1 апреля
                } else if (month <= 9) { // 3 квартал
                    localDateD2 = localDateP2 = LocalDate.of(localDateD1.getYear(), 9, 30); // 30 сентября
                    localDateP1 = LocalDate.of(localDateD1.getYear(), 7, 1);; // 1 июля
                } else if (month <= 12) { // 4 квартал
                    localDateD2 = localDateP2 = LocalDate.of(localDateD1.getYear(), 12, 31); // 31 декабря
                    localDateP1 = LocalDate.of(localDateD1.getYear(), 10, 1);; // 1 октября
                }
                if(localDateD2.isAfter(end)){   // если временной интервал Уже, чем юнит
                    localDateP2 = localDateD2;
                    localDateD2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);  // записываем первый квартал из интервала
                    break;
                }
                setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);  // записываем первый квартал из интервала
                for (localDateD1 = localDateD2.plusDays(1); localDateD1.isBefore(end); localDateD1 = localDateD1.plusMonths(3)) {
                    localDateP1 = localDateD1;
                    localDateD2 = localDateP2 = localDateD1.plusMonths(3).minusDays(1);
                    if (localDateD2.isAfter(end)) {
                        localDateP2 = localDateD2;
                        localDateD2 = end;
                    }
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);
                }
                if(localDateD1.isEqual(end)){
                    localDateD1 = localDateD2 = localDateP2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);    // добавляем последний день квартала
                }
                break;

            case YEAR:
                localDateD1 = start;
                localDateP1 = start.withDayOfYear(1); // первый день года
                localDateD2 = localDateP2 = localDateD1.withDayOfYear(1).plusYears(1).minusDays(1);
                if(localDateD2.isAfter(end)){   // если временной интервал Уже, чем юнит
                    localDateP2 = localDateD2;
                    localDateD2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);  // записываем первый год из интервала
                    break;
                }
                setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);  // записываем первый год из интервала
                for (localDateD1 = localDateD2.plusDays(1); localDateD1.isBefore(end); localDateD1 = localDateD1.plusYears(1)) {
                    localDateP1 = localDateD1;
                    localDateD2 = localDateP2 = localDateD1.plusYears(1).minusDays(1);
                    if (localDateD2.isAfter(end)) {
                        localDateP2 = localDateD2;
                        localDateD2 = end;
                    }
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);
                }
                if(localDateD1.isEqual(end)){
                    localDateD1 = localDateD2 = localDateP2 = end;
                    setOutputItem(result, localDateD1, localDateD2, localDateP1, localDateP2);    // добавляем последний день года
                }
                break;

            default:
        }

        return result;
    }


    // Заполнить элемент возвращаемого списка периодов
    private static void setOutputItem(List<ru.roe.timesheets.utils.Period> result, LocalDate localDateD1, LocalDate localDateD2, LocalDate localDateP1, LocalDate localDateP2) {
        Date dateResultD1;
        Date dateResultD2;
        Date dateResultP1;
        Date dateResultP2;
        ru.roe.timesheets.utils.Period period;
        dateResultD1 = java.sql.Date.valueOf(localDateD1);  // приведение LocalDate к Date
        dateResultD2 = java.sql.Date.valueOf(localDateD2);    // приведение LocalDate к Date
        dateResultP1 = java.sql.Date.valueOf(localDateP1);    // приведение LocalDate к Date
        dateResultP2 = java.sql.Date.valueOf(localDateP2);    // приведение LocalDate к Date
//        System.out.println("[" + dateResultP1);
//        System.out.println("с: " + dateResultD1 + " по: " + dateResultD2);
//        System.out.println(dateResultP2 + "]");
//        System.out.println("---");
        period = new ru.roe.timesheets.utils.Period();
        period.setStartDate(dateResultD1);
        period.setEndDate(dateResultD2);
        period.setStartPeriodDate(dateResultP1);
        period.setEndPeriodDate(dateResultP2);
        result.add(period);
    }


}