package ru.roe.timesheets.service;

import com.google.common.base.Strings;
import com.haulmont.cuba.core.Persistence;
import com.haulmont.cuba.core.Query;
import com.haulmont.cuba.core.entity.KeyValueEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.roe.timesheets.entity.*;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@Service(DimensionService.NAME)
public class DimensionServiceBean implements DimensionService {

    @Inject
    private Persistence persistence;

    @Override
    @Transactional ("MSProject")
    public List<MspProject> getProjectList(MspProjectType filterProjectType, Integer filterProjectYear, String filterNamePart) {
        String wsWhere = "";

        if (filterProjectType != null) {
            if (wsWhere != "")
                wsWhere = wsWhere + " and ";
            wsWhere = wsWhere + "([MSP_EpmCPPrjUid0].[CFVal2] = '" + filterProjectType.getOwnId() + "')";
        }

        if (filterProjectYear != null)
        if (filterProjectYear.intValue() > 0) {
            if (wsWhere != "")
                wsWhere = wsWhere + " and ";
            wsWhere = wsWhere + "(CAST(MSP_EpmLookupTable14.MemberFullValue AS nvarchar(4)) = '" + filterProjectYear + "')";
        }

        if (!Strings.isNullOrEmpty(filterNamePart))
        {
            if (wsWhere != "")
                wsWhere = wsWhere + " and ";
            wsWhere = wsWhere + "[MSP_EpmProject].[ProjectName] like '%" + filterProjectYear + "%')";
        }

        String wsSQL = "SELECT  [MSP_EpmProject].[ProjectUID] , [MSP_EpmProject].[ProjectName] , [MSP_EpmProject].[ProjectTitle], [MSP_EpmCPPrjUid0].[CFVal2] as project_type_id, [1. Вид проекта]=CAST (MSP_EpmLookupTable13.MemberFullValue AS nvarchar(4000)) , [2. Год проекта]=CAST(MSP_EpmLookupTable14.MemberFullValue AS nvarchar(4)) "
         +" FROM dbo.[MSP_EpmProject] "
         +" LEFT OUTER JOIN dbo.[MSP_EpmCPPrjUid0] ON [MSP_EpmCPPrjUid0].EntityUID = MSP_EpmProject.ProjectUID "
         +" LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable13 ON MSP_EpmLookupTable13.LookupTableUID = 'FE1CB9E3-778D-4F51-A9C7-F2F6C2ED1B6E' AND MSP_EpmLookupTable13.MemberUID=[MSP_EpmCPPrjUid0].[CFVal2] "
         +" LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable14 ON MSP_EpmLookupTable14.LookupTableUID = 'F67D4030-E857-4204-BE21-433673C72FAD' AND MSP_EpmLookupTable14.MemberUID=[MSP_EpmCPPrjUid0].[CFVal3] "
         +" WHERE ([MSP_EpmCPPrjUid0].[CFVal2] is not null) "
         +" order by CAST(MSP_EpmLookupTable14.MemberFullValue AS nvarchar(4)) desc, [MSP_EpmProject].[ProjectName]";

        if (wsWhere != "")
            wsSQL = wsSQL + " and " + wsWhere;

        System.out.println("projectList: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        List list = query.getResultList();

        List<MspProject> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                MspProject item = new MspProject();
                item.setOwnId((String) row[0]);
                item.setProjectName((String) row[1]);
                item.setProjectTitle((String) row[2]);

                if (row[3] != null) {
                    String projectTypeId = (String) row[3];
                    String projectTypeName = (String) row[4];

                    MspProjectType projectType = new MspProjectType();
                    projectType.setOwnId(projectTypeId);
                    projectType.setShortName(projectTypeName);
                    projectType.setCompleteName(projectTypeName);
                    item.setProjectType(projectType);
                }

                if (row[5] != null) {
                    String projectYear = (String) row[5];
                    Integer intProjectYear = Integer.parseInt(projectYear);
                    item.setProjectYear(intProjectYear);
                }
                result.add(item);
            }
        }
        return result;
    }

    @Override
    @Transactional ("MSProject")
    public List<MspProjectType> getProjectTypeList() {
        String wsSQL = "select memberUID, MemberValue, MemberFullValue from MSP_EpmLookupTable where LookupTableUID = 'FE1CB9E3-778D-4F51-A9C7-F2F6C2ED1B6E' and MemberValue is not null order by MemberValue";
        System.out.println("projectTypeList: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        List list = query.getResultList();

        List<MspProjectType> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                MspProjectType item = new MspProjectType();
                item.setOwnId((String) row[0]);
                item.setShortName((String) row[1]);
                item.setCompleteName((String) row[2]);
                result.add(item);
            }
        }
        return result;
    }

    @Override
    @Transactional ("MSProject")
    public List<MspLifeCyclePart> getLifeCycleParts() {
        String wsSQL = "select memberUID, MemberValue, MemberFullValue from MSP_EpmLookupTable where LookupTableUID = '19F91BC5-35D2-430F-B98F-0E2BFBA99C99' and MemberValue is not null order by MemberValue";
        System.out.println("LifeCyclePartList: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        List list = query.getResultList();

        List<MspLifeCyclePart> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                MspLifeCyclePart item = new MspLifeCyclePart();
                item.setOwnId((String) row[0]);
                item.setShortName((String) row[1]);
                item.setCompleteName((String) row[2]);
                result.add(item);
            }
        }
        return result;
    }

    @Override
    @Transactional ("MSProject")
    public List<MspResource> getResources() {
        String wsSQL = "select resourceUID, replace(resourceNtAccount,'RVNET\\','') as rv_code, resourceName from [MSP_EpmResource] where resourceGroup = '0521' and resourceisactive = 1 order by resourceName";
        System.out.println("ResourcesList: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        List list = query.getResultList();

        List<MspResource> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                MspResource item = new MspResource();
                item.setOwnId((String) row[0]);
                item.setAccount((String) row[1]);
                item.setName((String) row[2]);
                result.add(item);
            }
        }
        return result;
    }

    @Override
    @Transactional ("MSProject")
    public List<MspSystem> getSystems() {
        String wsSQL = "select mspSystem.memberUID, mspSystem.MemberValue, mspSystem.MemberFullValue, mspSystem.ParentMemberUID "
         +"from MSP_EpmLookupTable mspSystem "
         +"where mspSystem.LookupTableUID = '7B1E149C-621A-4AC5-A496-2642B8A04230' and memberValue is not null "
         +"order by mspSystem.ParentMemberUID, mspSystem.MemberValue";
        System.out.println("SystemList: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        List list = query.getResultList();

        List<MspSystem> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                MspSystem item = new MspSystem();
                item.setOwnId((String) row[0]);
                item.setShortName((String) row[1]);
                item.setCompleteName((String) row[2]);
                if (row[3] != null) {
                    for (MspSystem searchItem : result) {
                        if (searchItem.getOwnId().equals((String) row[3]))
                            item.setParentLink(searchItem);
                    }
                }
                result.add(item);
            }
        }
        return result;
    }

    @Override
    @Transactional ("MSProject")
    public List<MspWorkType> getWorkTypes() {
        String wsSQL = "select memberUID, MemberValue, MemberFullValue, ParentMemberUID from MSP_EpmLookupTable where LookupTableUID = 'A44F3EF1-83DF-4A9C-AECA-AB41B884D323' and MemberValue is not null order by ParentMemberUID, MemberValue";
        System.out.println("LifeCyclePartList: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        List list = query.getResultList();

        List<MspWorkType> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                MspWorkType item = new MspWorkType();
                item.setOwnId((String) row[0]);
                item.setShortName((String) row[1]);
                item.setCompleteName((String) row[2]);

                if (row[3] != null) {
                    for (MspWorkType searchItem : result) {
                        if (searchItem.getOwnId().equals((String) row[3]))
                            item.setParentLink(searchItem);
                    }
                }

                result.add(item);
            }
        }
        return result;
    }
}