package ru.roe.timesheets.service;

import com.google.common.base.Strings;
import com.haulmont.cuba.core.Persistence;
import com.haulmont.cuba.core.Query;
import com.haulmont.cuba.core.entity.KeyValueEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.roe.timesheets.core.BuildFilterStringBean;
import ru.roe.timesheets.core.TimeScaleBean;
import ru.roe.timesheets.entity.*;
import ru.roe.timesheets.enumeration.GroupingMethod;
import ru.roe.timesheets.enumeration.TimeScaleUnit;
import ru.roe.timesheets.utils.Period;

import javax.inject.Inject;
import java.math.BigDecimal;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service(FactOLAPService.NAME)
public class FactOLAPServiceBean implements FactOLAPService {

    @Inject
    private Persistence persistence;

    @Inject
    private BuildFilterStringBean buildFilterStringBean;

    @Transactional("MSProject")
    private List<KeyValueEntity> MakeBaseOLAPRequest(Boolean outputByCategory, GroupingMethod groupingMethod, TimeScaleUnit timeScaleUnit, Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, Boolean highLevelGrouping)
    {
        String wsWhere = buildFilterStringBean.getFilterString(startDate, endDate, filterProject, filterResource, filterSystem, filterWorkType, filterLifeCyclePart, null, groupingMethod, null, null);

        String wsGroupBy = "";
        String wsJoinAdd = "";
        switch (groupingMethod) {
            case BY_PROJECTS:
                wsGroupBy = "P.ProjectUID, P.ProjectName";
                break;
            case BY_SYSTEM:
                if (!highLevelGrouping)
                {
                    wsGroupBy = "MSP_EpmLookupTable3.MemberUID, MSP_EpmLookupTable3.memberValue";
                }
                    else
                {
                    wsGroupBy = "isnull(MSP_EpmLookupTable3parent.MemberUID, MSP_EpmLookupTable3.MemberUID), isnull(MSP_EpmLookupTable3parent.memberValue, MSP_EpmLookupTable3.memberValue)";
                    wsJoinAdd = "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable3parent ON (MSP_EpmLookupTable3parent.LookupTableUID = '7B1E149C-621A-4AC5-A496-2642B8A04230') AND (MSP_EpmLookupTable3.parentmemberUid = MSP_EpmLookupTable3parent.MemberUID) ";
                }
                break;
            case BY_PROJECT_TYPE:
                wsGroupBy = "[MSP_EpmCPPrjUid0].[CFVal2], MSP_EpmLookupTable13.MemberValue";
                break;
            case BY_RESOURCE:
                wsGroupBy = "R.ResourceUID, R.ResourceName";
                break;
            case BY_WORK_TYPE:
                if (!highLevelGrouping) {
                    wsGroupBy = "MSP_EpmLookupTable4.MemberUID, MSP_EpmLookupTable4.memberValue";
                }
                else
                {
                    wsGroupBy = "isnull(MSP_EpmLookupTable4parent.MemberUID, MSP_EpmLookupTable4.MemberUID), isnull(MSP_EpmLookupTable4parent.memberValue, MSP_EpmLookupTable4.memberValue)";
                    wsJoinAdd = "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable4parent ON (MSP_EpmLookupTable4parent.LookupTableUID = 'A44F3EF1-83DF-4A9C-AECA-AB41B884D323') AND (MSP_EpmLookupTable4.parentmemberUid = MSP_EpmLookupTable4parent.MemberUID) ";
                }

                break;
            case BY_LIFE_CYCLE_PART:
                wsGroupBy = "MSP_EpmLookupTable2.MemberUID, MSP_EpmLookupTable2.MemberValue";
                break;
        }
        String wsCategoryFields = wsGroupBy;
        List<Period> periodList = TimeScaleBean.getTimePeriodsByUnit(startDate, endDate, timeScaleUnit);

        String wsExactPeriodField = "";
        if (timeScaleUnit != null) {
            String wsGroupField = "";
            switch(timeScaleUnit) {
                case DAY:
                    wsGroupField = ", convert(varchar(10),AD.TimeByDay,102)";
                    wsExactPeriodField = "AD.TimeByDay";
                    break;
                case WEEK:
                    wsGroupField = ", convert(varchar(10),DATEADD(DAY,DATEDIFF(DAY,0,AD.TimeByDay)/7*7,0),102)";
                    wsExactPeriodField = "DATEADD(DAY,DATEDIFF(DAY,0,AD.TimeByDay)/7*7,0)";
                    break;
                case MONTH:
                    wsGroupField = ", convert(varchar(10),convert(date,'1.'+str(month(AD.TimeByDay))+'.'+str(year(AD.TimeByDay)),104),102)";
                    wsExactPeriodField = "convert(date,'1.'+str(month(AD.TimeByDay))+'.'+str(year(AD.TimeByDay)),104)";
                    break;
                case QUARTER:
                    wsGroupField = ", convert(varchar(10),convert(date,'1.'+str(((month(AD.TimeByDay) - 1) / 3) * 3 + 1)+'.'+str(year(AD.TimeByDay)),104),102)";
                    wsExactPeriodField = "convert(date,'1.'+str(((month(AD.TimeByDay) - 1) / 3) * 3 + 1)+'.'+str(year(AD.TimeByDay)),104)";
                    break;
                case YEAR:
                    wsGroupField = ", convert(varchar(10),convert(date,'1.1.'+str(year(AD.TimeByDay)),104),102)";
                    wsExactPeriodField = "convert(date,'1.1.'+str(year(AD.TimeByDay)),104)";
                    break;
                default:
            }
            if (!Strings.isNullOrEmpty(wsGroupField))
              wsGroupBy = wsGroupBy + wsGroupField + ", " + wsExactPeriodField;
        }


        String wsAgregate = "CONVERT(decimal(18, 1), SUM(AD.AssignmentActualWork)) AS Fact";

        String wsSQL = "SELECT " + wsGroupBy + ", " + wsAgregate + " " +
                "FROM MSP_EpmAssignmentByDay_UserView AS AD " +
                "INNER JOIN MSP_EpmAssignment AS A ON AD.AssignmentUID = A.AssignmentUID " +
                 "INNER JOIN [MSP_EpmResource] AS R ON A.ResourceUID = R.ResourceUID ";

        wsSQL = wsSQL + "INNER JOIN MSP_EpmTask AS T ON A.TaskUID = T.TaskUID " +
                "LEFT OUTER JOIN dbo.[MSP_EpmCPTaskUid0] ON [MSP_EpmCPTaskUid0].EntityUID = T.TaskUID " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable2 ON MSP_EpmLookupTable2.LookupTableUID = '19F91BC5-35D2-430F-B98F-0E2BFBA99C99' AND MSP_EpmLookupTable2.MemberUID=[MSP_EpmCPTaskUid0].[CFVal1] " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable3 ON MSP_EpmLookupTable3.LookupTableUID = '7B1E149C-621A-4AC5-A496-2642B8A04230' AND MSP_EpmLookupTable3.MemberUID=[MSP_EpmCPTaskUid0].[CFVal2] " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable4 ON MSP_EpmLookupTable4.LookupTableUID = 'A44F3EF1-83DF-4A9C-AECA-AB41B884D323' AND MSP_EpmLookupTable4.MemberUID=[MSP_EpmCPTaskUid0].[CFVal3] " +
                "left outer JOIN [MSP_EpmProject] AS P ON A.ProjectUID = P.ProjectUID ";

        wsSQL = wsSQL + "LEFT OUTER JOIN dbo.[MSP_EpmCPPrjUid0] ON [MSP_EpmCPPrjUid0].EntityUID = P.ProjectUID " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable13 ON MSP_EpmLookupTable13.LookupTableUID = 'FE1CB9E3-778D-4F51-A9C7-F2F6C2ED1B6E' AND MSP_EpmLookupTable13.MemberUID=[MSP_EpmCPPrjUid0].[CFVal2] " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable14 ON MSP_EpmLookupTable14.LookupTableUID = 'F67D4030-E857-4204-BE21-433673C72FAD' AND MSP_EpmLookupTable14.MemberUID=[MSP_EpmCPPrjUid0].[CFVal3] " +
                wsJoinAdd +
                wsWhere + " ";

        if (!Strings.isNullOrEmpty(wsGroupBy))
            wsSQL = wsSQL + " group by " + wsGroupBy;

        wsSQL = wsSQL + " HAVING (SUM(AD.AssignmentActualWork) <> 0) ";
        if (outputByCategory)
        {
            wsSQL = wsSQL + " ORDER BY " + wsCategoryFields;
        }
        else {
            if (timeScaleUnit != null) {
                wsSQL = wsSQL + " ORDER BY " + wsExactPeriodField;
            } else {
                wsSQL = wsSQL + " ORDER BY SUM(AD.AssignmentActualWork) desc";
            }
        }

        System.out.println("TableViewRequest: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        System.out.println("Request ready");
        List list = query.getResultList();
        System.out.println("Request completed");

        List<KeyValueEntity> result = new ArrayList<>();

        String storedPeriodKey = "";
        String storedCategoryId = "";

        KeyValueEntity currentItem = null;
        for (Object each : list) {
            Object[] row = (Object[]) each;

            int colCount = 0;
            String wsCategoryId = (String) row[colCount];
            colCount = colCount + 1;
            String wsCategoryName = (String) row[colCount];
            colCount = colCount + 1;
            String wsPeriod = "";
            Date startPeriodDate = null;
            if (timeScaleUnit != null) {
//                wsPeriod = (String) row[colCount];
                colCount = colCount + 1;
                startPeriodDate = (Date) row[colCount];
                colCount = colCount + 1;
                wsPeriod = TimeScaleBean.getPeriodName(timeScaleUnit, startPeriodDate);
            }
            BigDecimal factHours = (BigDecimal) row[colCount];

            if (outputByCategory)
            {
                if (!storedCategoryId.equals(wsCategoryId)) {
                    currentItem = new KeyValueEntity();
                    currentItem.setValue("categoryId", wsCategoryId);
                    currentItem.setValue("categoryName", wsCategoryName);
                    result.add(currentItem);
                    storedCategoryId = wsCategoryId;
                }

                if (currentItem != null) {
                    Date startAnchorDate = startPeriodDate;
                    if (timeScaleUnit == TimeScaleUnit.WEEK)
                    {
                        for (Period periodItem : periodList)
                        {
                          if ((periodItem.getStartPeriodDate().compareTo(startAnchorDate) <= 0) && (periodItem.getEndPeriodDate().compareTo(startAnchorDate) > 0))
                            {
                                startAnchorDate = periodItem.getStartPeriodDate();
                                break;
                            }
                        }
                    }
                    Format formatter = new SimpleDateFormat("dd.MM.yyyy");
                    String wsPeriodStartDate = formatter.format(startAnchorDate);
                    currentItem.setValue("p_" + wsPeriodStartDate, factHours);
                }
            }
            else
            {
                if (timeScaleUnit == null) {
                    KeyValueEntity item = new KeyValueEntity();
                    item.setValue("categoryId", wsCategoryId);
                    item.setValue("categoryName", wsCategoryName);
                    item.setValue("factHours", factHours);
                    result.add(item);
                } else {
                    if (!storedPeriodKey.equals(wsPeriod)) {
                        currentItem = new KeyValueEntity();
                        currentItem.setValue("partLabel", wsPeriod);
                        currentItem.setValue("startPeriodDate", startPeriodDate);
                        Date endPeriodDate = TimeScaleBean.getEndDate(timeScaleUnit, startPeriodDate);
                        currentItem.setValue("endPeriodDate", endPeriodDate);
                        result.add(currentItem);
                        storedPeriodKey = wsPeriod;
                    }

                    if (currentItem != null)
                        currentItem.setValue("f_" + wsCategoryId, factHours);
                }
            }
        }

        if ((outputByCategory) && (groupingMethod == GroupingMethod.BY_RESOURCE))
        {
            wsSQL = "SELECT R.ResourceUID, R.ResourceName from MSP_EpmResource R where (ResourceIsActive = 1) ";
            wsSQL = wsSQL + "and (R.ResourceName <> 'Универсальный локальный ресурс') ";
            wsSQL = wsSQL + "and (R.ResourceName <> 'Неизвестный ресурс') ";
            wsSQL = wsSQL + "and (R.ResourceName <> 'Неназначенный ресурс') ";
            wsSQL = wsSQL + "and (R.ResourceName <> 'System administrator') ";
            query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
            list = query.getResultList();
            for (Object each : list) {
                Object[] row = (Object[]) each;
                Boolean itemInResultSet = false;
            for (KeyValueEntity item : result)
            {
                String categoryId = (String) item.getValue("categoryId");
                if (categoryId.equals((String) row[0]))
                {
                    itemInResultSet = true;
                    break;
                }
            }
            if (!itemInResultSet)
            {
                KeyValueEntity additionalItem = new KeyValueEntity();
                additionalItem.setValue("categoryId", (String) row[0]);
                additionalItem.setValue("categoryName", (String) row[1]);
                result.add(additionalItem);
            }
            }
        }

        System.out.println("Service ready");

        return result;
    }

    @Override
    @Transactional("MSProject")
    public List<KeyValueEntity> getFactOLAPStructure(GroupingMethod groupingMethod, Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, Boolean highLevelGrouping) {
        return MakeBaseOLAPRequest(false, groupingMethod, null, startDate, endDate, filterProject, filterResource, filterSystem, filterWorkType, filterLifeCyclePart, highLevelGrouping);
    }

    @Override
    @Transactional("MSProject")
    public List<KeyValueEntity> getFactOLAPDynamics(GroupingMethod groupingMethod, TimeScaleUnit timeScaleUnit, Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, Boolean highLevelGrouping) {
        return MakeBaseOLAPRequest(false,groupingMethod, timeScaleUnit, startDate, endDate, filterProject, filterResource, filterSystem, filterWorkType, filterLifeCyclePart, highLevelGrouping);
    }

    @Override
    @Transactional("MSProject")
    public List<KeyValueEntity> getFactControlDynamics(GroupingMethod groupingMethod, TimeScaleUnit timeScaleUnit, Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, Boolean highLevelGrouping) {
        return MakeBaseOLAPRequest(true,groupingMethod, timeScaleUnit, startDate, endDate, filterProject, filterResource, filterSystem, filterWorkType, filterLifeCyclePart, highLevelGrouping);
    }
}