package ru.roe.timesheets.service;

import com.google.common.base.Strings;
import com.haulmont.cuba.core.Persistence;
import com.haulmont.cuba.core.Query;
import com.haulmont.cuba.core.entity.KeyValueEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.roe.timesheets.core.BuildFilterStringBean;
import ru.roe.timesheets.entity.*;
import ru.roe.timesheets.enumeration.GroupingMethod;

import javax.inject.Inject;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service(FactViewService.NAME)
public class FactViewServiceBean implements FactViewService {

    @Inject
    private Persistence persistence;

    @Inject
    private BuildFilterStringBean buildFilterStringBean;

    @Transactional("MSProject")
    private List getFactViewRecords(Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, String filterTaskNamePart, GroupingMethod groupingMethod, Boolean exactFilter, Boolean nullFilter)
    {
        String wsWhere = buildFilterStringBean.getFilterString(startDate, endDate, filterProject, filterResource, filterSystem, filterWorkType, filterLifeCyclePart, filterTaskNamePart, groupingMethod, exactFilter, nullFilter);

        String wsSQL = "SELECT P.ProjectUID, P.ProjectName, P.ProjectTitle, [MSP_EpmCPPrjUid0].[CFVal2] as work_type_code, MSP_EpmLookupTable13.MemberValue as work_type, MSP_EpmLookupTable14.MemberValue as projectYear, T.TaskUID, T.TaskName, T.TaskPercentCompleted, " +
                "R.ResourceUID, R.ResourceName, replace(R.resourceNtAccount,'RVNET\','') as rv_code, CONVERT(decimal(18, 1), SUM(AD.AssignmentActualWork)) AS Fact, " + // --AD.TimeByDay,
                "MSP_EpmLookupTable2.MemberUID as work_type_code, MSP_EpmLookupTable2.MemberValue as work_type_name, MSP_EpmLookupTable3.MemberUID as system_code, MSP_EpmLookupTable3.memberValue as system_name, MSP_EpmLookupTable4.MemberUID as life_cycle_part_code, MSP_EpmLookupTable4.memberValue as life_cycle_part_name " +
                "FROM MSP_EpmAssignmentByDay_UserView AS AD " +
                "INNER JOIN MSP_EpmAssignment AS A ON AD.AssignmentUID = A.AssignmentUID " +
                "INNER JOIN [MSP_EpmResource] AS R ON A.ResourceUID = R.ResourceUID " +
                "INNER JOIN MSP_EpmTask AS T ON A.TaskUID = T.TaskUID " +
                "LEFT OUTER JOIN dbo.[MSP_EpmCPTaskUid0] ON [MSP_EpmCPTaskUid0].EntityUID = T.TaskUID " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable2 ON MSP_EpmLookupTable2.LookupTableUID = '19F91BC5-35D2-430F-B98F-0E2BFBA99C99' AND MSP_EpmLookupTable2.MemberUID=[MSP_EpmCPTaskUid0].[CFVal1] " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable3 ON MSP_EpmLookupTable3.LookupTableUID = '7B1E149C-621A-4AC5-A496-2642B8A04230' AND MSP_EpmLookupTable3.MemberUID=[MSP_EpmCPTaskUid0].[CFVal2] " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable4 ON MSP_EpmLookupTable4.LookupTableUID = 'A44F3EF1-83DF-4A9C-AECA-AB41B884D323' AND MSP_EpmLookupTable4.MemberUID=[MSP_EpmCPTaskUid0].[CFVal3] " +

                "left outer JOIN [MSP_EpmProject] AS P ON A.ProjectUID = P.ProjectUID " +

                "LEFT OUTER JOIN dbo.[MSP_EpmCPPrjUid0] ON [MSP_EpmCPPrjUid0].EntityUID = P.ProjectUID " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable13 ON MSP_EpmLookupTable13.LookupTableUID = 'FE1CB9E3-778D-4F51-A9C7-F2F6C2ED1B6E' AND MSP_EpmLookupTable13.MemberUID=[MSP_EpmCPPrjUid0].[CFVal2] " +
                "LEFT OUTER JOIN MSP_EpmLookupTable as MSP_EpmLookupTable14 ON MSP_EpmLookupTable14.LookupTableUID = 'F67D4030-E857-4204-BE21-433673C72FAD' AND MSP_EpmLookupTable14.MemberUID=[MSP_EpmCPPrjUid0].[CFVal3] " +

                wsWhere + " " +
                "GROUP BY T.TaskOutlineNumber, T.TaskUID, R.ResourceUID, P.ProjectUID, P.ProjectName, P.ProjectTitle, [MSP_EpmCPPrjUid0].[CFVal2], MSP_EpmLookupTable13.MemberValue, MSP_EpmLookupTable14.MemberValue, " +
                "R.resourceNtAccount, T.TaskName, R.ResourceName,  MSP_EpmLookupTable2.MemberUID, MSP_EpmLookupTable2.MemberValue, MSP_EpmLookupTable3.MemberUID, MSP_EpmLookupTable3.memberValue, MSP_EpmLookupTable4.MemberUID, MSP_EpmLookupTable4.memberValue, " +
                "T.TaskPercentCompleted " + //--, AD.TimeByDay
                "HAVING (SUM(AD.AssignmentActualWork) <> 0) " +
                "order by SUM(AD.AssignmentActualWork) desc";
//                "ORDER BY R.ResourceName, P.ProjectName, T.TaskOutlineNumber";

        System.out.println("TableViewRequest: " + wsSQL);
        Query query = persistence.getEntityManager("MSProject").createNativeQuery(wsSQL);
        //query.setParameter(1, "%Company%");
        System.out.println("Request ready");
        return query.getResultList();

    }



    @Override
    @Transactional("MSProject")
    public List<TableLine> getTableFactlines(Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, String filterTaskNamePart, GroupingMethod groupingMethod, Boolean exactFilter, Boolean nullFilter) {

        List list = getFactViewRecords(startDate, endDate, filterProject, filterResource, filterSystem, filterWorkType, filterLifeCyclePart, filterTaskNamePart, groupingMethod, exactFilter, nullFilter);

        List<TableLine> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                TableLine item = new TableLine();
                item.setProjectName((String) row[1]);
                item.setTaskName((String) row[7]);
                item.setResourceName((String) row[10]);
                item.setSystemName((String) row[16]);
                item.setLifeCyclePartName((String) row[18]);
                item.setWorkTypeName((String) row[14]);
                item.setPercentComplete(Integer.valueOf((Short) row[8]));
                item.setFactHours((BigDecimal) row[12]);
                item.setOwnLink(item);
                result.add(item);
            }
        }
        System.out.println("Service ready");

        return result;
    }

    @Override
    @Transactional("MSProject")
    public List<KeyValueEntity> getTableFactRows(Date startDate, Date endDate, MspProject filterProject, MspResource filterResource, MspSystem filterSystem, MspWorkType filterWorkType, MspLifeCyclePart filterLifeCyclePart, String filterTaskNamePart, GroupingMethod groupingMethod, Boolean exactFilter, Boolean nullFilter) {
        List list = getFactViewRecords(startDate, endDate, filterProject, filterResource, filterSystem, filterWorkType, filterLifeCyclePart, filterTaskNamePart, groupingMethod, exactFilter, nullFilter);

        List<KeyValueEntity> result = new ArrayList<>();

        for (Object each : list) {
            Object[] row = (Object[]) each;

            if (row[0] != null) {
                KeyValueEntity item = new KeyValueEntity();
                item.setValue("projectName",(String) row[1]);
                item.setValue("taskName",(String) row[7]);
                item.setValue("resourceName", (String) row[10]);
                item.setValue("systemName", (String) row[16]);
                item.setValue("lifeCyclePartName",(String) row[18]);
                item.setValue("workTypeName", (String) row[14]);
                item.setValue("percentComplete", Integer.valueOf((Short) row[8]));
                item.setValue("factHours", (BigDecimal) row[12]);
                result.add(item);
            }
        }
        System.out.println("Service ready");

        return result;
    }

}