package ru.roe.timesheets.web.screens.dynamicsreport;

import com.google.common.base.Strings;
import com.haulmont.charts.gui.amcharts.model.Graph;
import com.haulmont.charts.gui.amcharts.model.GraphType;
import com.haulmont.charts.gui.components.charts.Chart;
import com.haulmont.charts.gui.components.charts.SerialChart;
import com.haulmont.charts.gui.data.DataItem;
import com.haulmont.cuba.core.entity.KeyValueEntity;
import com.haulmont.cuba.gui.components.*;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.KeyValueCollectionContainer;
import com.haulmont.cuba.gui.model.KeyValueCollectionLoader;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.Subscribe;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import ru.roe.timesheets.entity.*;
import ru.roe.timesheets.enumeration.GroupingMethod;
import ru.roe.timesheets.enumeration.TimeScaleUnit;
import ru.roe.timesheets.service.DimensionService;
import ru.roe.timesheets.service.FactOLAPService;
import ru.roe.timesheets.service.FactViewService;

import javax.inject.Inject;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

import static ru.roe.timesheets.web.screens.projectreport.ProjectReport.DEFAULT_STYLE_NAME;
import static ru.roe.timesheets.web.screens.projectreport.ProjectReport.TEXT_WRAPPING_BY_WORDS_STYLE;

@UiController("timesheets_DynamicsReport")
@UiDescriptor("dynamics-report.xml")
public class DynamicsReport extends Screen {

    public static final String UNDEFINED_WORD = "уточняется";
    public static final int ASYNC_SEARCH_DELAY_MS = 300;    // задержка перед поиском (мс.)
    public static final int MIN_SEARCH_STRING_LENGTH = 2;   // минимальная длина запроса
    public static final int SUGGESTIONS_LIMIT = 100;        // лимит подсказок
    public static final String ROE_FIELD_EMPTY = "roe-field-empty";
    public static final String ROE_FIELD_NOT_EMPTY = "roe-field-not-empty";
    private String sliceCategoryId;        // id категории выделенного сегмента круговой диаграммы
    private String sliceCategoryName;   // имя категориии выделенного сегмента курговой диаграммы
    private Date startSegmentDate = null;
    private Date endSegmentDate = null;
    private String storedSliceCategoryId = "";        // хранимый с последнего нажатия id категории выделенного сегмента круговой диаграммы
    private String storedGraphId = "";
    private Boolean sliceCategoryIsNull = false;

    @Inject
    private CollectionLoader<MspProject> ProjectsDl;
    @Inject
    private DimensionService dimensionService;
    @Inject
    private SuggestionPickerField<MspProject> projectFilter;
    @Inject
    private CollectionContainer<MspProject> ProjectsDc;
    @Inject
    private CollectionLoader<MspResource> mspResourcesDl;
    @Inject
    private CollectionContainer<MspResource> mspResourcesDc;
    @Inject
    private SuggestionPickerField<MspResource> resourceFilter;
    @Inject
    private FactOLAPService factOLAPService;
    @Inject
    private LookupField groupingMethod;
    @Inject
    private DateField<Date> startDate;
    @Inject
    private DateField<Date> endDate;
    @Inject
    private CollectionLoader<MspSystem> mspSystemsDl;
    @Inject
    private SuggestionPickerField<MspSystem> systemFilter;
    @Inject
    private CollectionContainer<MspSystem> mspSystemsDc;
    @Inject
    private CollectionLoader<MspWorkType> mspWorkTypesDl;
    @Inject
    private SuggestionPickerField<MspWorkType> workTypeFilter;
    @Inject
    private CollectionContainer<MspWorkType> mspWorkTypesDc;
    @Inject
    private CollectionContainer<MspLifeCyclePart> mspLifeCyclePartsDc;
    @Inject
    private SuggestionPickerField<MspLifeCyclePart> lifeCyclePartFilter;
    @Inject
    private CollectionLoader<MspLifeCyclePart> mspLifeCyclePartsDl;
    @Inject
    private Label<String> totalSum;
    @Inject
    private CheckBox highLevelGrouping;
    @Inject
    private KeyValueCollectionLoader factDataByPeriodsDl;
    @Inject
    private LookupField timeScaleUnit;
    @Inject
    private KeyValueCollectionContainer factDataByPeriodsDc;
    @Inject
    private SerialChart factDataByPeriodsChart;
    @Inject
    private KeyValueCollectionLoader structureDataDl;
    @Inject
    private KeyValueCollectionContainer structureDataDc;
    @Inject
    private CollectionContainer<TableLine> TableLineDc;
    @Inject
    private Label<String> detailsLabel;
    @Inject
    private DataGrid<TableLine> detailsDataGrid;
    @Inject
    private CollectionLoader<TableLine> TableLineDl;
    @Inject
    private FactViewService factViewService;


    // EVENTS *****************************************************************************************************

    @Subscribe
    public void onInit(InitEvent event) {
        projectFilterInit();
        resourceFilterInit();
        systemFilterInit();
        workTypesFilterInit();
        lifeCyclePartFilterInit();
        groupingMethodLookupFieldInit(groupingMethod);  // инициализация выпадающего списка методов группировки
        setDefaults();
        getFactData();
    }

    @Subscribe("groupingMethod")
    public void onGroupingMethodValueChange(HasValue.ValueChangeEvent event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        switch ((GroupingMethod) groupingMethod.getValue()) {
            case BY_SYSTEM:
            case BY_WORK_TYPE:
                highLevelGrouping.setVisible(true);
                break;
            case BY_PROJECTS:
            case BY_RESOURCE:
            case BY_PROJECT_TYPE:
            case BY_LIFE_CYCLE_PART:
                highLevelGrouping.setVisible(false);
                break;
            default:
                break;
        }
    }

    @Subscribe("timeScaleUnit")
    public void onTimeScaleUnitValueChange(HasValue.ValueChangeEvent event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    @Subscribe("highLevelGrouping")
    public void onHighLevelGroupingValueChange(HasValue.ValueChangeEvent<Boolean> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    @Subscribe("projectFilter")
    public void onProjectFilterValueChange(HasValue.ValueChangeEvent<MspProject> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(projectFilter);
    }

    @Subscribe("resourceFilter")
    public void onResourceFilterValueChange(HasValue.ValueChangeEvent<MspResource> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(resourceFilter);
    }

    @Subscribe("systemFilter")
    public void onSystemFilterValueChange(HasValue.ValueChangeEvent<MspSystem> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(systemFilter);
    }

    @Subscribe("workTypeFilter")
    public void onWorkTypeFilterValueChange(HasValue.ValueChangeEvent<MspWorkType> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(workTypeFilter);
    }

    @Subscribe("lifeCyclePartFilter")
    public void onLifeCyclePartFilterValueChange(HasValue.ValueChangeEvent<MspLifeCyclePart> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(lifeCyclePartFilter);
    }

    @Subscribe("startDate")
    public void onStartDateValueChange(HasValue.ValueChangeEvent<Date> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    @Subscribe("endDate")
    public void onEndDateValueChange(HasValue.ValueChangeEvent<Date> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    // клик по сегменту диаграммы "По периодам"
    @Subscribe("factDataByPeriodsChart")
    private void onFactDataByPeriodsChartItemClick(Chart.GraphItemClickEvent event) {
        String graphId = event.getGraphId();
        sliceCategoryId = "";
        sliceCategoryName = "";
        List<String> graphIds = new ArrayList<>();
        List<Graph> graphs = factDataByPeriodsChart.getGraphs();
        for (Graph graph : graphs) {
            String wsGraphId = graph.getId();
            String wsCategoryId = graph.getValueField();
            if (wsCategoryId.indexOf("f_") == 0) {
                graphIds.add(wsCategoryId.substring(2));
                if (wsGraphId.equals(graphId)) {
                    sliceCategoryId = wsCategoryId.substring(2);
                    sliceCategoryName = graph.getTitle();
                }
            }
        }
        final DataItem selectedItem = event.getDataItem();
        if (selectedItem != null) {
            startSegmentDate = (Date) selectedItem.getValue("startPeriodDate");
            endSegmentDate = (Date) selectedItem.getValue("endPeriodDate");
        } else {
            startSegmentDate = startDate.getValue();
            endSegmentDate = endDate.getValue();
        }
        List<KeyValueEntity> items = factDataByPeriodsDc.getItems();
        // обработка выделения элемента диаграммы (изменение прозрачности графов диаграммы)
        setSelectedItemAlfas(items, selectedItem, sliceCategoryId, graphIds, "partLabel", "selectedAlfa_");
        event.getSource().repaint();
        // загрузка данных по выделенному сегменту
        String periodId = (String) selectedItem.getValue("partLabel");
        if ((storedSliceCategoryId.equals(sliceCategoryId)) && (storedGraphId.equals(periodId))) {
            sliceCategoryId = "";
            sliceCategoryName = "";
            startSegmentDate = startDate.getValue();
            endSegmentDate = endDate.getValue();
        }

        // обработка сегмента с незаполненными корпоративными полями
        if (sliceCategoryId.toUpperCase().equals("NULL")) {
            sliceCategoryIsNull = true;
        } else {
            sliceCategoryIsNull = false;
        }

        loadSliceData();    // загрузка данных по выделенному сегменту
        storedSliceCategoryId = sliceCategoryId;
        storedGraphId = periodId;
    }

    @Subscribe("chartRefresh")
    public void onChartRefreshClick(Button.ClickEvent event) {
        getFactData();
    }


// METHODS ****************************************************************************************************

    // Обработчик фильтра "Проект" (загрузка данных для поиска)
    private void projectFilterInit() {
        ProjectsDl.setLoadDelegate(valueLoadContext -> dimensionService.getProjectList(null, null, null));
        ProjectsDl.load();
        projectFilter.setEnabled(true);
        projectFilter.setSearchExecutor((searchString, searchParams) ->
                ProjectsDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        projectFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        projectFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        projectFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }

    // Обработчик фильтра "Исполнитель" (загрузка данных для поиска)
    private void resourceFilterInit() {
        mspResourcesDl.setLoadDelegate(valueLoadContext -> dimensionService.getResources());
        mspResourcesDl.load();
        resourceFilter.setEnabled(true);
        resourceFilter.setSearchExecutor((searchString, searchParams) ->
                mspResourcesDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        resourceFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        resourceFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        resourceFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }

    // Обработчик фильтра "Система" (загрузка данных для поиска)
    private void systemFilterInit() {
        mspSystemsDl.setLoadDelegate(valueLoadContext -> dimensionService.getSystems());
        mspSystemsDl.load();
        systemFilter.setEnabled(true);
        systemFilter.setSearchExecutor((searchString, searchParams) ->
                mspSystemsDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        systemFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        systemFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        systemFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }

    // Обработчик фильтра "Вид работ" (загрузка данных для поиска)
    private void workTypesFilterInit() {
        mspWorkTypesDl.setLoadDelegate(valueLoadContext -> dimensionService.getWorkTypes());
        mspWorkTypesDl.load();
        workTypeFilter.setEnabled(true);
        workTypeFilter.setSearchExecutor((searchString, searchParams) ->
                mspWorkTypesDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        workTypeFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        workTypeFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        workTypeFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }

    // Обработчик фильтра "Стадия ЖЦ" (загрузка данных для поиска)
    private void lifeCyclePartFilterInit() {
        mspLifeCyclePartsDl.setLoadDelegate(valueLoadContext -> dimensionService.getLifeCycleParts());
        mspLifeCyclePartsDl.load();
        lifeCyclePartFilter.setEnabled(true);
        lifeCyclePartFilter.setSearchExecutor((searchString, searchParams) ->
                mspLifeCyclePartsDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        lifeCyclePartFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        lifeCyclePartFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        lifeCyclePartFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }


    // прочитать данные для построения диаграмм
    private void getFactData() {
        buildFactDataByPeriodsStackedColumnChart();
        factDataByPeriodsDl.setLoadDelegate(valueLoadContext -> factOLAPService.getFactOLAPDynamics(
                (GroupingMethod) groupingMethod.getValue(),
                (TimeScaleUnit) timeScaleUnit.getValue(),
                startDate.getValue(),
                endDate.getValue(),
                projectFilter.getValue(),
                resourceFilter.getValue(),
                systemFilter.getValue(),
                workTypeFilter.getValue(),
                lifeCyclePartFilter.getValue(),
                highLevelGrouping.getValue()));
        factDataByPeriodsDl.load();

        // выводим общую сумму
        final BigDecimal sum = structureDataDc.getItems().stream()
                .filter(keyValueEntity -> keyValueEntity.getValue("factHours") != null)
                .map(keyValueEntity -> (BigDecimal) keyValueEntity.getValue("factHours"))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        DecimalFormat decimalFormat = new DecimalFormat();
        decimalFormat.setGroupingUsed(true);
        decimalFormat.setGroupingSize(3);
        BigDecimal daysCount = sum.divide(BigDecimal.valueOf(8), RoundingMode.HALF_UP);
        BigDecimal monthCount = daysCount.divide(BigDecimal.valueOf(21), RoundingMode.HALF_UP);
        totalSum.setValue("Суммарные трудозатраты: <span style='color:#14284B; text-decoration:underline'>" + decimalFormat.format(sum) + "</span> час. (" + decimalFormat.format(daysCount) + " дней или " + decimalFormat.format(monthCount) + " мес.)");

        // начитываем детали по всему блину
        sliceCategoryId = "";
        startSegmentDate = startDate.getValue();
        endSegmentDate = endDate.getValue();
        loadSliceData();  // детали трудозатрат
    }


    // инициализация выпадающего списка методов группировки
    private static void groupingMethodLookupFieldInit(LookupField groupingMethod) {
        List<GroupingMethod> groupingMethodList = new ArrayList<>();
        groupingMethodList.add(GroupingMethod.BY_PROJECTS);
        groupingMethodList.add(GroupingMethod.BY_RESOURCE);
        groupingMethodList.add(GroupingMethod.BY_SYSTEM);
        groupingMethodList.add(GroupingMethod.BY_PROJECT_TYPE);
        groupingMethodList.add(GroupingMethod.BY_WORK_TYPE);
        groupingMethodList.add(GroupingMethod.BY_LIFE_CYCLE_PART);
        groupingMethod.setOptionsList(groupingMethodList);
    }


    // установить значения фильтров по умолчанию
    private void setDefaults() {
        // интервал дат
        LocalDate monthBeginning = LocalDate.now().withDayOfMonth(1);
        Date d1 = java.sql.Date.valueOf(monthBeginning);  // первый день месяца;
        Date d2 = new Date();    // сегодня
        startDate.setValue(d1);
        endDate.setValue(d2);
        // метод группировки
        groupingMethod.setValue(GroupingMethod.BY_SYSTEM);
        highLevelGrouping.setVisible(true);
        highLevelGrouping.setValue(true);
        // временной интервал
        timeScaleUnit.setValue(TimeScaleUnit.WEEK);
        // развернуть оси графика
        factDataByPeriodsChart.getCategoryAxis().setLabelRotation(45); // развернуть подписи на осях
        // стили для фильтров
        setPickerStyle(projectFilter);
        setPickerStyle(resourceFilter);
        setPickerStyle(systemFilter);
        setPickerStyle(workTypeFilter);
        setPickerStyle(lifeCyclePartFilter);
    }


    // установить стиль для заполненного фильтра
    private void setPickerStyle(SuggestionPickerField suggestionPickerField) {
        if (suggestionPickerField.getValue() != null) {
            suggestionPickerField.removeStyleName(ROE_FIELD_EMPTY);
            suggestionPickerField.addStyleName(ROE_FIELD_NOT_EMPTY);
        } else {
            suggestionPickerField.removeStyleName(ROE_FIELD_NOT_EMPTY);
            suggestionPickerField.addStyleName(ROE_FIELD_EMPTY);
        }
    }


    // построить Stacked Column Chart "По периодам"
    private void buildFactDataByPeriodsStackedColumnChart() {
        factDataByPeriodsDc.getMutableItems().clear();
        factDataByPeriodsDc.addProperty("partLabel", String.class);
        factDataByPeriodsDc.addProperty("selected", String.class);
        factDataByPeriodsDc.addProperty("startPeriodDate", Date.class);
        factDataByPeriodsDc.addProperty("endPeriodDate", Date.class);
        factDataByPeriodsChart.setGraphs(null);
        factDataByPeriodsChart.setSizeFull();

        structureDataDl.setLoadDelegate(valueLoadContext -> factOLAPService.getFactOLAPStructure(
                (GroupingMethod) groupingMethod.getValue(),
                startDate.getValue(),
                endDate.getValue(),
                projectFilter.getValue(),
                resourceFilter.getValue(),
                systemFilter.getValue(),
                workTypeFilter.getValue(),
                lifeCyclePartFilter.getValue(),
                highLevelGrouping.getValue()));
        structureDataDl.load();

        for (KeyValueEntity item : structureDataDc.getItems()) {
            String categoryId = item.getValue("categoryId");
            String graphId = "f_" + categoryId;
            String categoryName = item.getValue("categoryName");
            if (categoryId == null) {
                categoryId = "NULL";    // соглашение с сервисом...
                categoryName = "Не заполнено";
            }
                // строим graph и добавляем его в factDataByPeriodsChart
                Graph graph = new Graph();
                graph.setId(graphId);
                graph.setFillAlphas(0.8);
                graph.setType(GraphType.COLUMN);
                graph.setLineAlpha(0.2);
                graph.setAlphaField("selectedAlfa_" + categoryId);  // для изменения прозрачности выделенного сегмента диаграммы
                graph.setTitle(categoryName);
                graph.setBehindColumns(true);
                graph.setValueField(graphId);
                graph.setBalloonText("[[category]] - " + categoryName + ": <strong>[[value]]<strong>");
                factDataByPeriodsChart.addGraphs(graph);
                factDataByPeriodsDc.addProperty("graphId", String.class);
        }
    }


    // обработка выделения элемента диаграммы (изменение прозрачности графов диаграммы)
    public static void setSelectedItemAlfas(List<KeyValueEntity> items, DataItem selectedItem, String sliceCategoryId, List<String> graphIds, String segmentIdPropertyName, String alfaPropertyNamePrefix) {
        Double alphaValue = Double.valueOf(0.8); // прозрачность по умолчанию
        Double selectedItemAlphaValue = Double.valueOf(0.95);
        Double notSelectedItemsAlphaValue = Double.valueOf(0.2);
        Boolean hasSelectedItem = false;

        String segmentId = (String) selectedItem.getValue(segmentIdPropertyName);
        if (selectedItem.getValue(alfaPropertyNamePrefix + sliceCategoryId) != null) {
            if ((Double) selectedItem.getValue(alfaPropertyNamePrefix + sliceCategoryId) >= selectedItemAlphaValue) {
                hasSelectedItem = true;
            }
        }
        if (hasSelectedItem) {
            alphaValue = null;
        } else {
            alphaValue = notSelectedItemsAlphaValue;
        }
        for (KeyValueEntity item : items) {
            if ((String) item.getValue(segmentIdPropertyName) == segmentId) {
                if ((selectedItem.getValue(alfaPropertyNamePrefix + sliceCategoryId) == null) ||
                        ((Double) selectedItem.getValue(alfaPropertyNamePrefix + sliceCategoryId) < Double.valueOf(0.4))) {
                    for (String currentGraphId : graphIds) {
                        item.setValue(alfaPropertyNamePrefix + currentGraphId, alphaValue);
                    }
                    item.setValue(alfaPropertyNamePrefix + sliceCategoryId, selectedItemAlphaValue);
                } else {
                    for (String currentGraphId : graphIds) {
                        item.setValue(alfaPropertyNamePrefix + currentGraphId, alphaValue);
                    }
                }
            } else {
                for (String currentGraphId : graphIds) {
                    item.setValue(alfaPropertyNamePrefix + currentGraphId, alphaValue);
                }
            }
        }

    }


    // собрать данные для детализации
    private void loadSliceData() {
        Date dateFrom = startSegmentDate;
        Date dateTo = endSegmentDate;
        MspProject curProject = projectFilter.getValue();
        MspResource curResource = resourceFilter.getValue();
        MspSystem curSystem = systemFilter.getValue();
        MspWorkType curWorkType = workTypeFilter.getValue();
        MspLifeCyclePart curLifeCyclePart = lifeCyclePartFilter.getValue();

        GroupingMethod grMethod = (GroupingMethod) groupingMethod.getValue();

        // для выбора "чистого ПК-ЭС"
        Boolean ef = false;
        if (!highLevelGrouping.isChecked()) {
            ef = true;
        }
        final Boolean exactFilter = ef;

        if ( !(Strings.isNullOrEmpty(sliceCategoryId) || (sliceCategoryId.toUpperCase().equals("NULL"))) ) {
            switch (grMethod) {
                case BY_PROJECTS:
                    curProject = new MspProject();
                    curProject.setOwnId(sliceCategoryId);
                    break;
                case BY_RESOURCE:
                    curResource = new MspResource();
                    curResource.setOwnId(sliceCategoryId);
                    break;
                case BY_SYSTEM:
                    curSystem = new MspSystem();
                    curSystem.setOwnId(sliceCategoryId);
                    break;
                case BY_WORK_TYPE:
                    curWorkType = new MspWorkType();
                    curWorkType.setOwnId(sliceCategoryId);
                    break;
                case BY_LIFE_CYCLE_PART:
                    curLifeCyclePart = new MspLifeCyclePart();
                    curLifeCyclePart.setOwnId(sliceCategoryId);
                    break;
                default:
                    break;
            }
        }

        // загрузка данных Проектов
        final MspProject prj = curProject;
        final MspLifeCyclePart lcp = curLifeCyclePart;
        final MspWorkType wt = curWorkType;
        final MspSystem sys = curSystem;
        final MspResource res = curResource;
        final Boolean sliceCategoryIsNullFinal = sliceCategoryIsNull;
        sliceCategoryIsNull = false; // обнуляем

        TableLineDl.setLoadDelegate(valueLoadContext ->
                factViewService.getTableFactlines(dateFrom, dateTo, prj, res, sys, wt, lcp, null, grMethod, exactFilter, sliceCategoryIsNullFinal));
        TableLineDl.load();

        // отрисовка HTML таблиц в dataGrid
        getDetachmentDetailsHtmlString();
        // стиль отображения колонки dataGrid
        detailsDataGrid.getColumn("ownLink").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            }
            return TEXT_WRAPPING_BY_WORDS_STYLE;
        });
        // установка поясняющего текста над Details (SliceDetailsLabel)
        setSliceDetailsLabelText(grMethod);

    }


    // оформление data grid
    private void getDetachmentDetailsHtmlString() {
        getDetailsTableHeaderHtmlString();    // строим заголовок таблицы контрактов
        // html рендеринг колонок DataGrid
        detailsDataGrid.getColumn("ownLink").setRenderer(
                detailsDataGrid.createRenderer(DataGrid.HtmlRenderer.class),
                (Function<TableLine, String>) content -> {
                    DecimalFormat decimalFormat = new DecimalFormat();
                    decimalFormat.setGroupingUsed(true);
                    decimalFormat.setGroupingSize(3);
                    StringBuilder sb = new StringBuilder();
                    String htmlString = "";
                    BigDecimal fact = BigDecimal.ZERO;
                    String taskName = "";
                    String projectName = "";
                    String resourceName = "";
                    String systemName = "";
                    // Отображаемые данные
                    if (content != null) {
                        if (content.getTaskName() != null)
                            taskName = content.getTaskName(); // задача
                        if (content.getProjectName() != null)
                            projectName = content.getProjectName(); // задача
                        if (content.getResourceName() != null)
                            resourceName = content.getResourceName(); // исполнитель
                        if (content.getSystemName() != null)
                            systemName = content.getSystemName(); // система
                        if (content.getFactHours() != null)
                            fact = content.getFactHours(); // факт

                        // сборка строки HTML для ячейки таблицы
                        htmlString = htmlString + "<table width='100%' align='center'>";
                        htmlString = htmlString + "<tr style='font-size:12px'>";
                        htmlString = htmlString + "<td width='33%'>" + resourceName + "</td>";
                        htmlString = htmlString + "<td width='33%'>" + fact + "</td>";
                        htmlString = htmlString + "<td>" + systemName + "</td>";
                        htmlString = htmlString + "</tr>";
                        htmlString = htmlString + "<tr>";
                        htmlString = htmlString + "<td colspan=4 style='font-size:11px'>";

                        // формирование HTML для отображения области DETAILS сроки
                        sb.append(" <b>Работа:</b> ");
                        sb.append(taskName);
                        sb.append(" <br><b>Проект:</b> ");
                        sb.append(projectName);
                        sb.append(".");

                        htmlString = htmlString + sb.toString();
                        htmlString = htmlString + "</td>";
                        htmlString = htmlString + "</tr>";
                        htmlString = htmlString + "</table>";
                    } else
                        htmlString = UNDEFINED_WORD;
                    return htmlString;
                }
        );
    }


    // строим заголовок таблицы детализации
    private void getDetailsTableHeaderHtmlString() {
        String htmlHeaderString = "";
        htmlHeaderString = htmlHeaderString + "<table width='100%' align='center'>";
        htmlHeaderString = htmlHeaderString + "<tr style='font-size:12px; font-weight:bold'>";
        htmlHeaderString = htmlHeaderString + "<td width='33%'>Исполнитель</td>";
        htmlHeaderString = htmlHeaderString + "<td width='33%'>Часы</td>";
        htmlHeaderString = htmlHeaderString + "<td>Система</td>";
        htmlHeaderString = htmlHeaderString + "</tr>";
        detailsDataGrid.getDefaultHeaderRow().getCell("ownLink").setHtml(htmlHeaderString);
    }


    // Установка поясняющего текста над Details (SliceDetailsLabel)
    private void setSliceDetailsLabelText(GroupingMethod grMethod) {
        String detailsText;    // результирующий текст
        // сумма
        final BigDecimal sum = TableLineDc.getItems().stream()
                .filter(keyValueEntity -> keyValueEntity.getValue("factHours") != null)
                .map(keyValueEntity -> (BigDecimal) keyValueEntity.getValue("factHours"))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        DecimalFormat decimalFormat = new DecimalFormat();
        decimalFormat.setGroupingUsed(true);
        decimalFormat.setGroupingSize(3);
        BigDecimal daysCount = sum.divide(BigDecimal.valueOf(8), RoundingMode.HALF_UP);
        BigDecimal monthCount = daysCount.divide(BigDecimal.valueOf(21), RoundingMode.HALF_UP);

        if (sliceCategoryId == null || (sliceCategoryId.toUpperCase().equals("NULL")) ) {
            detailsText = "Аналитическое поле не заполнено: " + decimalFormat.format(sum) + " часов (" + decimalFormat.format(daysCount) + " дней или " + decimalFormat.format(monthCount) + " мес.)";
            ;
        } else {
            if (!sliceCategoryId.equals("")) {
                String category = "";   // категория
                switch (grMethod) {
                    case BY_RESOURCE:
                        category = TableLineDc.getItems().get(0).getResourceName();
                        break;
                    case BY_SYSTEM:
                        category = TableLineDc.getItems().get(0).getSystemName();
                        break;
                    case BY_PROJECTS:
                        category = TableLineDc.getItems().get(0).getProjectName();
                        break;
                    case BY_WORK_TYPE:
                        category = TableLineDc.getItems().get(0).getWorkTypeName();
                        break;
                    case BY_LIFE_CYCLE_PART:
                        category = TableLineDc.getItems().get(0).getLifeCyclePartName();
                        break;
                    default:
                        break;
                }
                detailsText = category + ": " + decimalFormat.format(sum) + " часов (" + decimalFormat.format(daysCount) + " дней или " + decimalFormat.format(monthCount) + " мес.)";
            } else {
                detailsText = "Всего: " + decimalFormat.format(sum) + " часов (" + decimalFormat.format(daysCount) + " дней или " + decimalFormat.format(monthCount) + " мес.)";
            }
        }
        detailsLabel.setValue(detailsText);
    }


}