package ru.roe.timesheets.web.screens.tablereport;

import com.haulmont.cuba.gui.components.*;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.screen.*;
import ru.roe.timesheets.entity.*;
import ru.roe.timesheets.service.DimensionService;
import ru.roe.timesheets.service.FactViewService;
import javax.inject.Inject;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.util.Collection;
import java.util.Date;
import java.util.stream.Collectors;

@UiController("timesheets_TableReport")
@UiDescriptor("table-report.xml")
public class TableReport extends Screen {

    public static final int ASYNC_SEARCH_DELAY_MS = 300;    // задержка перед поиском (мс.)
    public static final int MIN_SEARCH_STRING_LENGTH = 2;   // минимальная длина запроса
    public static final int SUGGESTIONS_LIMIT = 100;        // лимит подсказок
    public static final String ROE_FIELD_EMPTY = "roe-field-empty";
    public static final String ROE_FIELD_NOT_EMPTY = "roe-field-not-empty";

    public static final String DEFAULT_STYLE_NAME = "default-style";
    public static final String TEXT_WRAPPING_BY_WORDS_STYLE = "text-wrapping-by-words";
    public static final String SUBTOTALS_STYLE_NAME = "header-row-totals";

    DataGrid.HeaderRow headerRow;


    @Inject
    private FactViewService factViewService;
    @Inject
    private DimensionService dimensionService;
    @Inject
    private CollectionLoader<TableLine> TableLineDl;
    @Inject
    private CollectionLoader<MspProject> ProjectsDl;
    @Inject
    private SuggestionPickerField<MspProject> projectFilter;
    @Inject
    private CollectionContainer<MspProject> ProjectsDc;
    @Inject
    private CollectionLoader<MspResource> mspResourcesDl;
    @Inject
    private CollectionContainer<MspResource> mspResourcesDc;
    @Inject
    private SuggestionPickerField<MspResource> resourceFilter;
    @Inject
    private DateField<Date> startDate;
    @Inject
    private DateField<Date> endDate;
    @Inject
    private CollectionLoader<MspSystem> mspSystemsDl;
    @Inject
    private SuggestionPickerField<MspSystem> systemFilter;
    @Inject
    private CollectionContainer<MspSystem> mspSystemsDc;
    @Inject
    private CollectionLoader<MspWorkType> mspWorkTypesDl;
    @Inject
    private SuggestionPickerField<MspWorkType> workTypeFilter;
    @Inject
    private CollectionContainer<MspWorkType> mspWorkTypesDc;
    @Inject
    private CollectionContainer<MspLifeCyclePart> mspLifeCyclePartsDc;
    @Inject
    private SuggestionPickerField<MspLifeCyclePart> lifeCyclePartFilter;
    @Inject
    private CollectionLoader<MspLifeCyclePart> mspLifeCyclePartsDl;
    @Inject
    private DataGrid<TableLine> ReportLinesDataGrid;
    @Inject
    private CollectionContainer<TableLine> TableLineDc;
    @Inject
    private Label<String> totalSum;


    //** EVENTS ****************************************************************************

    @Subscribe
    public void onInit(InitEvent event) {
        projectFilterInit();
        resourceFilterInit();
        systemFilterInit();
        workTypesFilterInit();
        lifeCyclePartFilterInit();
        setDefaults();
        initHeader();
        dataGridColumnStyling();
        getFactData();
    }

    @Subscribe("projectFilter")
    public void onProjectFilterValueChange(HasValue.ValueChangeEvent<MspProject> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(projectFilter);
    }

    @Subscribe("resourceFilter")
    public void onResourceFilterValueChange(HasValue.ValueChangeEvent<MspResource> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(resourceFilter);
    }

    @Subscribe("systemFilter")
    public void onSystemFilterValueChange(HasValue.ValueChangeEvent<MspSystem> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(systemFilter);
    }

    @Subscribe("workTypeFilter")
    public void onWorkTypeFilterValueChange(HasValue.ValueChangeEvent<MspWorkType> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(workTypeFilter);
    }

    @Subscribe("lifeCyclePartFilter")
    public void onLifeCyclePartFilterValueChange(HasValue.ValueChangeEvent<MspLifeCyclePart> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
        setPickerStyle(lifeCyclePartFilter);
    }

    @Subscribe("startDate")
    public void onStartDateValueChange(HasValue.ValueChangeEvent<Date> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    @Subscribe("endDate")
    public void onEndDateValueChange(HasValue.ValueChangeEvent<Date> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }




// METHODS ****************************************************************************************************

    // Обработчик фильтра "Проект" (загрузка данных для поиска)
    private void projectFilterInit() {
        ProjectsDl.setLoadDelegate(valueLoadContext -> dimensionService.getProjectList(null, null, null));
        ProjectsDl.load();
        projectFilter.setEnabled(true);
        projectFilter.setSearchExecutor((searchString, searchParams) ->
                ProjectsDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        projectFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        projectFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        projectFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }
    // Обработчик фильтра "Исполнитель" (загрузка данных для поиска)
    private void resourceFilterInit() {
        mspResourcesDl.setLoadDelegate(valueLoadContext -> dimensionService.getResources());
        mspResourcesDl.load();
        resourceFilter.setEnabled(true);
        resourceFilter.setSearchExecutor((searchString, searchParams) ->
                mspResourcesDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        resourceFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        resourceFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        resourceFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }
    // Обработчик фильтра "Система" (загрузка данных для поиска)
    private void systemFilterInit() {
        mspSystemsDl.setLoadDelegate(valueLoadContext -> dimensionService.getSystems());
        mspSystemsDl.load();
        systemFilter.setEnabled(true);
        systemFilter.setSearchExecutor((searchString, searchParams) ->
                mspSystemsDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        systemFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        systemFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        systemFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }
    // Обработчик фильтра "Вид работ" (загрузка данных для поиска)
    private void workTypesFilterInit() {
        mspWorkTypesDl.setLoadDelegate(valueLoadContext -> dimensionService.getWorkTypes());
        mspWorkTypesDl.load();
        workTypeFilter.setEnabled(true);
        workTypeFilter.setSearchExecutor((searchString, searchParams) ->
                mspWorkTypesDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        workTypeFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        workTypeFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        workTypeFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }
    // Обработчик фильтра "Стадия ЖЦ" (загрузка данных для поиска)
    private void lifeCyclePartFilterInit() {
        mspLifeCyclePartsDl.setLoadDelegate(valueLoadContext -> dimensionService.getLifeCycleParts());
        mspLifeCyclePartsDl.load();
        lifeCyclePartFilter.setEnabled(true);
        lifeCyclePartFilter.setSearchExecutor((searchString, searchParams) ->
                mspLifeCyclePartsDc.getItems().stream()
                        .filter(inc -> inc.complexSearch(searchString))
                        .collect(Collectors.toList()));
        lifeCyclePartFilter.setAsyncSearchDelayMs(ASYNC_SEARCH_DELAY_MS); // задержка перед поиском (мс.)
        lifeCyclePartFilter.setMinSearchStringLength(MIN_SEARCH_STRING_LENGTH); // минимальная длина запроса
        lifeCyclePartFilter.setSuggestionsLimit(SUGGESTIONS_LIMIT); // лимит подсказок
    }


    // прочитать данные для построения диаграмм
    private void getFactData() {
        // загрузка данных Проектов
        TableLineDl.setLoadDelegate(valueLoadContext ->
                factViewService.getTableFactlines(
                        startDate.getValue(),
                        endDate.getValue(),
                        projectFilter.getValue() ,
                        resourceFilter.getValue(),
                        systemFilter.getValue(),
                        workTypeFilter.getValue(),
                        lifeCyclePartFilter.getValue(),
                        null,
                        null,
                        null,
                        null));
        TableLineDl.load();
        // подсчет итого
        headerRow.getCell("factHours").setText(getSum("factHours"));

        // выводим общую сумму
        final BigDecimal sum = TableLineDc.getItems().stream()
                .filter(keyValueEntity -> keyValueEntity.getValue("factHours") != null)
                .map(keyValueEntity -> (BigDecimal) keyValueEntity.getValue("factHours"))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        DecimalFormat decimalFormat = new DecimalFormat();
        decimalFormat.setGroupingUsed(true);
        decimalFormat.setGroupingSize(3);
        BigDecimal daysCount = sum.divide(BigDecimal.valueOf(8), RoundingMode.HALF_UP);
        BigDecimal monthCount = daysCount.divide(BigDecimal.valueOf(21), RoundingMode.HALF_UP);
        totalSum.setValue("Суммарные трудозатраты: <span style='color:#14284B; text-decoration:underline'>" + decimalFormat.format(sum) + "</span> час. (" + decimalFormat.format(daysCount) + " дней или " + decimalFormat.format(monthCount) + " мес.)");

    }

    // установить значения фильтров по умолчанию
    private void setDefaults() {
        // интервал дат
        LocalDate monthBeginning = LocalDate.now().minusWeeks(1); //.withDayOfMonth(1);
        Date d1 = java.sql.Date.valueOf(monthBeginning);  // первый день месяца;
        Date d2 = new Date();    // сегодня
        startDate.setValue(d1);
        endDate.setValue(d2);
        // установка стилей для фильтров
        setPickerStyle(projectFilter);
        setPickerStyle(resourceFilter);
        setPickerStyle(systemFilter);
        setPickerStyle(workTypeFilter);
        setPickerStyle(lifeCyclePartFilter);
    }


    // установить стиль для заполненного фильтра
    private void setPickerStyle(SuggestionPickerField suggestionPickerField) {
        if (suggestionPickerField.getValue() != null) {
            suggestionPickerField.removeStyleName(ROE_FIELD_EMPTY);
            suggestionPickerField.addStyleName(ROE_FIELD_NOT_EMPTY);
        } else {
            suggestionPickerField.removeStyleName(ROE_FIELD_NOT_EMPTY);
            suggestionPickerField.addStyleName(ROE_FIELD_EMPTY);
        }
    }


    private void initHeader() {
        headerRow = ReportLinesDataGrid.appendHeaderRow();
        headerRow.getCell("projectName").setHtml("<strong>" + "Итого:" + "</strong>");
        headerRow.getCell("taskName").setText("");
        headerRow.getCell("resourceName").setText("");
        headerRow.getCell("factHours").setText(getSum("factHours"));
        headerRow.getCell("percentComplete").setText("");
        headerRow.getCell("systemName").setText("");
        headerRow.getCell("lifeCyclePartName").setText("");
        headerRow.getCell("workTypeName").setText("");
        headerRow.setStyleName(SUBTOTALS_STYLE_NAME);
    }


    private String getSum(String propertyId) {
        BigDecimal sum = BigDecimal.ZERO;
        Collection<TableLine> items = TableLineDc.getItems();
        for (TableLine tableLine : items) {
            BigDecimal value = tableLine.getFactHours();
            sum = sum.add(value);
        }
        return sum.toString();
    }


    // стилизация колонок data grid
    private void dataGridColumnStyling() {
        ReportLinesDataGrid.getColumn("projectName").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });
        ReportLinesDataGrid.getColumn("taskName").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });
        ReportLinesDataGrid.getColumn("resourceName").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });
        ReportLinesDataGrid.getColumn("factHours").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });
        ReportLinesDataGrid.getColumn("percentComplete").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });
        ReportLinesDataGrid.getColumn("systemName").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });
        ReportLinesDataGrid.getColumn("lifeCyclePartName").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });
        ReportLinesDataGrid.getColumn("workTypeName").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            } else {
                return TEXT_WRAPPING_BY_WORDS_STYLE;
            }
        });


//        ReportLinesDataGrid.getColumn("period").setStyleProvider((property) -> {
//            if (property == null) {
//                return DEFAULT_STYLE_NAME;
//            }
//            if (!property.getPeriodDefined()) {
//                return TEXT_WRAPPING_ALL_STYLE_YELLOW;
//            }
//            return TEXT_WRAPPING_BY_WORDS_STYLE;
//        });


    }



}