package ru.roe.timesheets.web.screens.weeklycontrol;

import com.haulmont.cuba.core.entity.KeyValueEntity;
import com.haulmont.cuba.gui.screen.Screen;
import com.haulmont.cuba.gui.screen.UiController;
import com.haulmont.cuba.gui.screen.UiDescriptor;
import com.haulmont.cuba.gui.components.*;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.CollectionLoader;
import com.haulmont.cuba.gui.model.KeyValueCollectionLoader;
import com.haulmont.cuba.gui.screen.Subscribe;

import ru.roe.timesheets.entity.*;
import ru.roe.timesheets.enumeration.GroupingMethod;
import ru.roe.timesheets.enumeration.TimeScaleUnit;
import ru.roe.timesheets.service.FactOLAPService;
import ru.roe.timesheets.service.FactViewService;
import ru.roe.timesheets.service.TimeScaleService;
import ru.roe.timesheets.utils.Period;

import javax.inject.Inject;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


@UiController("timesheets_WeeklyControl")
@UiDescriptor("weekly-control.xml")
public class WeeklyControl extends Screen {

    public static final String DEFAULT_STYLE_NAME = "default-style";
    public static final String TEXT_WRAPPING_BY_WORDS_STYLE = "text-wrapping-by-words";
    public static final String DATA_GRID_CELL_STYLE_RED = "data-grid-cell-red";
    public static final String DATA_GRID_CELL_STYLE_GREY = "data-grid-cell-grey";
    public static final String UNDEFINED_WORD = "уточняется";
    private String sliceCategoryId;        // id категории выделенного сегмента круговой диаграммы
    private Date startSegmentDate = null;
    private Date endSegmentDate = null;

    @Inject
    private FactOLAPService factOLAPService;
    @Inject
    private DateField<Date> startDate;
    @Inject
    private DateField<Date> endDate;
    @Inject
    private LookupField timeScaleUnit;
    @Inject
    private CollectionContainer<TableLine> TableLineDc;
    @Inject
    private Label<String> detailsLabel;
    @Inject
    private DataGrid<TableLine> detailsDataGrid;
    @Inject
    private CollectionLoader<TableLine> TableLineDl;
    @Inject
    private FactViewService factViewService;
    @Inject
    private KeyValueCollectionLoader controlDataDl;
    @Inject
    private TimeScaleService timeScaleService;
    @Inject
    private DataGrid controlDataGrid;


    // EVENTS *****************************************************************************************************

    @Subscribe
    public void onInit(InitEvent event) {
        setDefaults();
        getFactData();
    }

    @Subscribe("startDate")
    public void onStartDateValueChange(HasValue.ValueChangeEvent<Date> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    @Subscribe("endDate")
    public void onEndDateValueChange(HasValue.ValueChangeEvent<Date> event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    @Subscribe("timeScaleUnit")
    public void onTimeScaleUnitValueChange(HasValue.ValueChangeEvent event) {
        if (!event.isUserOriginated()) return;
        getFactData();
    }

    @Subscribe("chartRefresh")
    public void onChartRefreshClick(Button.ClickEvent event) {
        getFactData();
    }

    @Subscribe("controlDataGrid")
    public void onControlDataGridItemClick(DataGrid.ItemClickEvent event) {
        sliceCategoryId = ((KeyValueEntity) event.getItem()).getValue("categoryId"); // controlDataDc
        startSegmentDate = startDate.getValue();
        endSegmentDate = endDate.getValue();
        if (sliceCategoryId != null) {
            loadSliceData();    // детали трудозатрат
        }
    }


// METHODS ****************************************************************************************************

    // прочитать данные для построения диаграмм
    private void getFactData() {
        removePeriodColumns();
        createPeriodColumns();

        controlDataDl.setLoadDelegate(valueLoadContext -> factOLAPService.getFactControlDynamics(
                GroupingMethod.BY_RESOURCE,
                (TimeScaleUnit) timeScaleUnit.getValue(),
                startDate.getValue(),
                endDate.getValue(),
                null,
                null,
                null,
                null,
                null,
                null));
        controlDataDl.load();

        // начитываем детали по всему блину
        sliceCategoryId = "";
        startSegmentDate = startDate.getValue();
        endSegmentDate = endDate.getValue();
        loadSliceData();  // детали трудозатрат
    }


    // установить значения фильтров по умолчанию
    private void setDefaults() {
        // интервал дат
        LocalDate previousFriday = LocalDate.now(ZoneId.of("America/Montreal")).with(TemporalAdjusters.previous(DayOfWeek.FRIDAY));
        LocalDate currentThursday = LocalDate.now(ZoneId.of("America/Montreal")).with(DayOfWeek.THURSDAY);
        Date d1 = java.sql.Date.valueOf(previousFriday);  // пятница предыдущей недели;
        Date d2 = java.sql.Date.valueOf(currentThursday);  // четверг текущей недели;
        startDate.setValue(d1);
        endDate.setValue(d2);
        // временной интервал
        timeScaleUnit.setValue(TimeScaleUnit.DAY);
    }


    // собрать данные для детализации
    private void loadSliceData() {
        Date dateFrom = startSegmentDate;
        Date dateTo = endSegmentDate;

        MspResource curResource;
        if (!sliceCategoryId.equals("")) {
            curResource = new MspResource();
            curResource.setOwnId(sliceCategoryId);
        } else {
            curResource = null;
        }
        final MspResource resource = curResource;

        TableLineDl.setLoadDelegate(valueLoadContext ->
                factViewService.getTableFactlines(
                        dateFrom,
                        dateTo,
                        null,
                        resource,
                        null,
                        null,
                        null,
                        null,
                        null,
                        null,
                        null));
        TableLineDl.load();

        // отрисовка HTML таблиц в dataGrid
        getDetailsHtmlString();
        // стиль отображения колонки dataGrid
        detailsDataGrid.getColumn("ownLink").setStyleProvider((property) -> {
            if (property == null) {
                return DEFAULT_STYLE_NAME;
            }
            return TEXT_WRAPPING_BY_WORDS_STYLE;
        });
        // установка поясняющего текста над Details (SliceDetailsLabel)
        setSliceDetailsLabelText();
    }


    // удалить сгенерированные колонки из DataGrid
    private void removePeriodColumns() {
        List<DataGrid.Column> columnList = controlDataGrid.getColumns();
        if (columnList != null) {
            if (columnList.size() > 1) {
                for (int i = columnList.size() - 1; i > 0; i--) {
                    controlDataGrid.removeColumn(columnList.get(i));
                }
            }
        }
    }


    // сгенерировать колонки в DataGrid
    private void createPeriodColumns() {
        List<Period> periodList = timeScaleService.getPeriodsByUnit(startDate.getValue(), endDate.getValue(), (TimeScaleUnit) timeScaleUnit.getValue());
        Boolean isVacation = false;
        for (Period p : periodList) {
            Format formatter = new SimpleDateFormat("dd.MM.yyyy");
            String periodColumnName = timeScaleService.getPeriodName((TimeScaleUnit) timeScaleUnit.getValue(), p.getStartDate());
            String periodId = "p_" + formatter.format(p.getStartPeriodDate());
            // обработка выходных дней
            if (p.getPeriodIsHoliday()) {
                isVacation = true;
            } else {
                isVacation = false;
            }
            final Boolean isVacationFinal = isVacation;

            // добавляем колонку
            controlDataGrid.addGeneratedColumn(periodId, new DataGrid.ColumnGenerator<KeyValueEntity, String>() {
                @Override
                public String getValue(DataGrid.ColumnGeneratorEvent<KeyValueEntity> event) {
                    KeyValueEntity item = event.getItem();
                    BigDecimal factHours = item.getValue(periodId);
                    DecimalFormat df = new DecimalFormat();
                    df.setMaximumFractionDigits(2);
                    df.setMinimumFractionDigits(0);
                    df.setGroupingUsed(false);
                    String result = "";
                    if (factHours != null) {
                        result = df.format(factHours);
                    }
                    return result;
                }

                @Override
                public Class<String> getType() {
                    return String.class;
                }
            }).setCaption(periodColumnName);

            // стилизация ячеек в столбцах
            controlDataGrid.getColumn(periodId).setStyleProvider((property) -> {
                if (property == null) {
                    return DEFAULT_STYLE_NAME;
                }
                BigDecimal factHours = ((KeyValueEntity) property).getValue(periodId);
                TimeScaleUnit curTimeScaleUnit = (TimeScaleUnit) timeScaleUnit.getValue();
                switch (curTimeScaleUnit){
                    case DAY:
                        if (factHours != null) {
                            if (!factHours.equals(BigDecimal.valueOf(8.0))) return DATA_GRID_CELL_STYLE_RED;
                        } else {
                            if (isVacationFinal) return DATA_GRID_CELL_STYLE_GREY;
                            else return DATA_GRID_CELL_STYLE_RED;
                        }
                        break;
                    case WEEK:
                        if (factHours != null) {
                            if (!factHours.equals(BigDecimal.valueOf(40.0))) return DATA_GRID_CELL_STYLE_RED;
                        } else {
                            if (isVacationFinal) return DATA_GRID_CELL_STYLE_GREY;
                            else return DATA_GRID_CELL_STYLE_RED;
                        }
                        break;
                    case MONTH:
                    case QUARTER:
                    case YEAR:
                    default:
                        break;
                }
                return DEFAULT_STYLE_NAME;
            });


        }
    }


    // оформление data grid
    private void getDetailsHtmlString() {
        getDetailsTableHeaderHtmlString();    // строим заголовок таблицы контрактов
        // html рендеринг колонок DataGrid
        detailsDataGrid.getColumn("ownLink").setRenderer(
                detailsDataGrid.createRenderer(DataGrid.HtmlRenderer.class),
                (Function<TableLine, String>) content -> {
                    DecimalFormat decimalFormat = new DecimalFormat();
                    decimalFormat.setGroupingUsed(true);
                    decimalFormat.setGroupingSize(3);
                    StringBuilder sb = new StringBuilder();
                    String htmlString = "";
                    BigDecimal fact = BigDecimal.ZERO;
                    String taskName = "";
                    String projectName = "";
                    String resourceName = "";
                    String systemName = "";
                    int percent = 0;
                    // Отображаемые данные
                    if (content != null) {
                        if (content.getPercentComplete() != null)
                            percent = content.getPercentComplete(); // задача
                        if (content.getTaskName() != null)
                            taskName = content.getTaskName(); // задача
                        if (content.getProjectName() != null)
                            projectName = content.getProjectName(); // задача
                        if (content.getResourceName() != null)
                            resourceName = content.getResourceName(); // исполнитель
                        if (content.getSystemName() != null)
                            systemName = content.getSystemName(); // система
                        if (content.getFactHours() != null)
                            fact = content.getFactHours(); // факт

                        // сборка строки HTML для ячейки таблицы
                        htmlString = htmlString + "<table width='100%' align='center'>";
                        htmlString = htmlString + "<tr style='font-size:12px'>";
                        htmlString = htmlString + "<td width='33%'>" + resourceName + "</td>";
                        htmlString = htmlString + "<td width='33%'>" + fact + "</td>";
                        htmlString = htmlString + "<td>" + systemName + "</td>";
                        htmlString = htmlString + "</tr>";
                        htmlString = htmlString + "<tr>";
                        htmlString = htmlString + "<td colspan=4 style='font-size:11px'>";

                        // формирование HTML для отображения области DETAILS сроки
                        sb.append(" <b>Работа:</b> ");
                        sb.append(taskName);
                        sb.append(" <br><b>Проект:</b> ");
                        sb.append(projectName);
                        sb.append(".");

                        htmlString = htmlString + sb.toString();
                        htmlString = htmlString + "</td>";
                        htmlString = htmlString + "</tr>";
                        htmlString = htmlString + "</table>";
                    } else
                        htmlString = UNDEFINED_WORD;
                    return htmlString;
                }
        );
    }


    // строим заголовок таблицы детализации
    private void getDetailsTableHeaderHtmlString() {
        String htmlHeaderString = "";
        htmlHeaderString = htmlHeaderString + "<table width='100%' align='center'>";
        htmlHeaderString = htmlHeaderString + "<tr style='font-size:12px; font-weight:bold'>";
        htmlHeaderString = htmlHeaderString + "<td width='33%'>Исполнитель</td>";
        htmlHeaderString = htmlHeaderString + "<td width='33%'>Часы</td>";
        htmlHeaderString = htmlHeaderString + "<td>Система</td>";
        htmlHeaderString = htmlHeaderString + "</tr>";
        detailsDataGrid.getDefaultHeaderRow().getCell("ownLink").setHtml(htmlHeaderString);
    }


    // Установка поясняющего текста над Details (SliceDetailsLabel)
    private void setSliceDetailsLabelText() {
        // сумма
        final BigDecimal sum = TableLineDc.getItems().stream()
                .filter(keyValueEntity -> keyValueEntity.getValue("factHours") != null)
                .map(keyValueEntity -> (BigDecimal) keyValueEntity.getValue("factHours"))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        DecimalFormat decimalFormat = new DecimalFormat();
        decimalFormat.setGroupingUsed(true);
        decimalFormat.setGroupingSize(3);
        BigDecimal daysCount = sum.divide(BigDecimal.valueOf(8), RoundingMode.HALF_UP);
        BigDecimal monthCount = daysCount.divide(BigDecimal.valueOf(21), RoundingMode.HALF_UP);

        String detailsText;    // результирующий текст

        if (TableLineDc.getItems().size() <= 0) {
            detailsText = "Детали отсутствуют";
        } else {
            if (!sliceCategoryId.equals("")) {
                String category = TableLineDc.getItems().get(0).getResourceName();  // категория
                detailsText = category + ": " + decimalFormat.format(sum) + " часов (" + decimalFormat.format(daysCount) + " дней или " + decimalFormat.format(monthCount) + " мес.)";
            } else {
                detailsText = "Всего: " + decimalFormat.format(sum) + " часов (" + decimalFormat.format(daysCount) + " дней или " + decimalFormat.format(monthCount) + " мес.)";
            }
        }
        detailsLabel.setValue(detailsText);
    }


}